import maya.cmds as cmds
import random

"""
Main execution function
"""
def makeMaterial(materialName):
    lambWarning = False
    
    #get selection
    selection = cmds.ls(selection=True)
    
    #set material type
    matType = cmds.optionMenu(materialType, query=True, value=True)
    if matType == "lambert" and cmds.checkBox(specCheck, query=True, value=True):
        matType = "blinn"
        lambWarning = True
    
    #get a list of selected properties
    checkboxes = [colorCheck, transCheck, specCheck]
    properties = ["color", "transparency", "specularColor"]
    selectedProperties = []
    for x in range(len(checkboxes)):
        if cmds.checkBox(checkboxes[x], query=True, value=True):
            selectedProperties.append(properties[x])
    
    #create shader, ramps, and switches then connect them all
    shader = cmds.shadingNode(matType, asShader=True, name=cmds.textField(materialName, query=True, text=True))
    ramps = []
    switches = []
    for x in range(len(selectedProperties)):
        ramps.append(cmds.shadingNode("ramp", asTexture=True, name=shader + "_" + selectedProperties[x]))
        switches.append(cmds.shadingNode("singleShadingSwitch", asUtility=True))
        cmds.connectAttr(ramps[x] + ".outColor", shader + "." + selectedProperties[x])
        cmds.connectAttr(switches[x] + ".output", ramps[x] + ".vCoord")
    
    for x in range(len(ramps)): #make the ramp black and white for transparency and specular by default
        if selectedProperties[x] != "color":
            cmds.setAttr(ramps[x] + ".colorEntryList[0].position", 0)
            cmds.setAttr(ramps[x] + ".colorEntryList[1].position", 1)
            cmds.setAttr(ramps[x] + ".colorEntryList[0].color", 0, 0, 0, type="double3")
            cmds.setAttr(ramps[x] + ".colorEntryList[1].color", 1, 1, 1, type="double3")
    
    #add coordinate attribute to objects and make the value random
    for x in range(len(selectedProperties)):
        curProp = selectedProperties[x] + "Coord"
        for object in selection:
            cmds.select(object)
            if cmds.objExists(object + "." + curProp):
                pass
            else:
                cmds.addAttr(object, longName=curProp, at="double", min=0, max=1, keyable=True)
            cmds.setAttr(object + "." + curProp, random.random())
            
    #apply the shader to the objects
    for object in selection:
        cmds.select(object)
        cmds.hyperShade(assign=shader)
        
    #link the objects and their colorCoord attributes to the shader switch
    for y in range(len(selectedProperties)):
        curProp = "." + selectedProperties[y] + "Coord"
        for x in range(len(selection)):
            selShape = cmds.listRelatives(selection[x])
            cmds.connectAttr(selShape[0] + ".instObjGroups[0]", switches[y] + ".input[" + str(x) + "].inShape", force=True)
            cmds.connectAttr(selection[x] + curProp, switches[y] + ".input[" + str(x) + "].inSingle", force=True)
    
    #select the objects again to maintain workflow
    cmds.select(selection)
    
    if lambWarning:
        cmds.warning("Lambert shaders don't have specular, so I made your material a Blinn instead.")
        
"""
Main window creation
"""
randomizeWindow = ""
if(cmds.window(randomizeWindow, exists=True)): #prevent multiple instances
    cmds.deleteUI(randomizeWindow)
randomizeWindow = cmds.window(title="Randomizer", sizeable=False) #create the window
cmds.columnLayout(adjustableColumn=True)

#==Help==#
cmds.rowLayout(numberOfColumns=3, adj=2)
cmds.button(label="Help", command="cmds.showHelp( 'http://www.alexberkowitz.wordpress.com/2013/09/15/revisiting-the-past-again/', absolute=True )")
cmds.button(vis=False) #I know it's not good to make layouts using hidden items but oh well.
cmds.button(label="About", command="cmds.showHelp( 'http://www.alexberkowitz.com/', absolute=True )")
cmds.setParent('..')

#==checkboxes==#
cmds.rowLayout(numberOfColumns=3)
colorCheck = cmds.checkBox(label="Color")
transCheck = cmds.checkBox(label="Transparency")
specCheck = cmds.checkBox(label="Specular Color")
cmds.setParent('..')

#==material options==#
cmds.rowLayout(numberOfColumns=2, adj=2) #material name
cmds.text(label="Material Name", width=70, align="right")
materialName = cmds.textField(text="random_mat")
cmds.setParent('..')
cmds.rowLayout(numberOfColumns=2, adj=2) #material type
cmds.text(label="Material Type", width=70, align="right")
materialType = cmds.optionMenu()
cmds.menuItem(label="blinn")
cmds.menuItem(label="lambert")
cmds.setParent('..')

#==start button==#
cmds.columnLayout(adjustableColumn=True)
cmds.button(label="Color my world!", command="makeMaterial(materialName)", height=50)
cmds.setParent('..')

cmds.showWindow(randomizeWindow) #show the window